# Copyright (C) 2009 Canonical Ltd
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from PyQt4 import QtCore, QtGui

from bzrlib import osutils
from bzrlib.commands import get_cmd_object

from bzrlib.plugins.explorer.lib import (
    # This is needed to load the right icon when run standalone
    explorer_rc,
    kinds,
    )
from bzrlib.plugins.explorer.lib.workspace_models import (
    MODEL_FEATURE_BRANCHES,
    MODEL_SHARED_TREE,
    MODEL_PLAIN_BRANCH,
    MODEL_SHARED_REPO,
    MODEL_SHARED_REPO_NO_TREES,
    )
from bzrlib.plugins.qbzr.lib.i18n import gettext
from bzrlib.plugins.qbzr.lib.init import fill_option_combo
from bzrlib.plugins.qbzr.lib.subprocess import SubProcessDialog


# Labels and tooltips for the workspace models
_MODEL_INFO = {
    MODEL_FEATURE_BRANCHES: (
        gettext("Feature branches"),
        gettext(
            "Create a shared repository at location then\n"
            "create a trunk branch inside that repository.")),
    MODEL_SHARED_TREE: (
        gettext("Shared tree"),
        gettext(
            "Create a shared repository with treeless branches at\n"
            "location. Then create a trunk branch and separate\n"
            "working tree (checkout) inside that repository.")),
    MODEL_PLAIN_BRANCH: (
        gettext("Plain branch"),
        gettext(
            "Create a plain branch at location. If inside a\n"
            "shared repository, revisions will be stored there.\n"
            "Otherwise the revisions will be stored in the branch.")),
    MODEL_SHARED_REPO: (
        gettext("Shared repository"),
        gettext(
            "Create a repository at location. Branches created\n"
            "under the repository will use it for storing revisions.")),
    MODEL_SHARED_REPO_NO_TREES: (
        gettext("Shared repository with treeless branches"),
        gettext(
            "Create a shared repository at location and configure it\n"
            "so branches under it are created without a working tree.\n"
            "Repositories on servers are usually configured this way.")),
    }


class QInitWorkspaceDialog(SubProcessDialog):

    def __init__(self, location, model=None, create_parent_dirs=False,
        dialog=True, ui_mode=True, parent=None):
        super(QInitWorkspaceDialog, self).__init__(
                                  gettext("Initialize"),
                                  name="init-ala-explorer",
                                  default_size=(400, 400),
                                  ui_mode=ui_mode,
                                  dialog=dialog,
                                  parent=parent,
                                  hide_progress=True,
                                  )
        if location == u'.':
            location = osutils.abspath(u'.')
        self._location = location
        self._create_parent_dirs = create_parent_dirs

        # Build the form
        layout = QtGui.QVBoxLayout(self)
        layout.addLayout(self._build_location_layout())
        layout.addWidget(self._build_model_group_box())
        layout.addWidget(self._build_format_group_box())
        layout.addWidget(self._build_options_group_box())
        layout.addWidget(self.make_default_status_box())
        layout.addWidget(self.buttonbox)

        # Init the controls
        if location:
            self.location.setText(location)
        if model:
            self.set_model(model)
        if create_parent_dirs:
            self.create_parent_dirs.setCheckState(QtCore.Qt.Checked)
        self.location.setFocus()
 
    def _build_location_layout(self):
        label = QtGui.QLabel(gettext("Location:"))
        self.location = QtGui.QLineEdit()
        browse_button = QtGui.QPushButton(gettext("Browse"))
        QtCore.QObject.connect(browse_button, QtCore.SIGNAL("clicked(bool)"),
            self._do_browse_clicked)
        layout = QtGui.QHBoxLayout()
        layout.addWidget(label)
        layout.addWidget(self.location)
        layout.addWidget(browse_button)
        return layout

    def _do_browse_clicked(self):
        location = self.location.text()
        fileName = QtGui.QFileDialog.getExistingDirectory(self,
            gettext("Select location"), location);
        if fileName:
            self.location.setText(fileName)

    def _build_model_group_box(self):
        layout = QtGui.QVBoxLayout()
        msg = kinds.html_status_message(kinds.GOOD_STATUS, gettext(
            "For more information about each model, see the tooltip."))
        tooltip_hint = QtGui.QLabel(msg)
        layout.addWidget(tooltip_hint)
        button_group = QtGui.QButtonGroup()
        self.feature_branches = self._model_radio(layout, button_group,
            MODEL_FEATURE_BRANCHES)
        self.feature_branches.setChecked(QtCore.Qt.Checked)
        self.shared_tree = self._model_radio(layout, button_group,
            MODEL_SHARED_TREE)
        self.plain_branch = self._model_radio(layout, button_group,
            MODEL_PLAIN_BRANCH)
        self.shared_repo = self._model_radio(layout, button_group,
            MODEL_SHARED_REPO)
        self.shared_repo_no_trees = self._model_radio(layout, button_group,
            MODEL_SHARED_REPO_NO_TREES)
        group_box = QtGui.QGroupBox(gettext("Model"))
        group_box.setLayout(layout)
        return group_box

    def _model_radio(self, layout, button_group, model_id):
        label, tooltip = _MODEL_INFO[model_id]
        radio =  QtGui.QRadioButton(label)
        radio.setToolTip(tooltip)
        layout.addWidget(radio)
        button_group.addButton(radio)
        return radio

    def _build_format_group_box(self):
        # Build the radio buttons line
        # TODO: perhaps only add '2.0' if available so users running
        # on Bazaar 1.14 to 1.16 get a better UI?
        button_group = QtGui.QButtonGroup()
        self.format_2 = QtGui.QRadioButton(gettext("2.0 default"))
        button_group.addButton(self.format_2)
        self.format_1 = QtGui.QRadioButton(gettext("1.x default"))
        button_group.addButton(self.format_1)
        self.format_other = QtGui.QRadioButton(gettext("Other"))
        button_group.addButton(self.format_other)
        self.format_2.setChecked(True)
        layout = QtGui.QHBoxLayout()
        layout.addWidget(self.format_2)
        layout.addWidget(self.format_1)
        layout.addWidget(self.format_other)

        # Build the other format field and control when it get displayed
        self.format_name = QtGui.QComboBox()
        self.format_name.setVisible(False)
        cmd = get_cmd_object('init')
        opt = cmd.options()['format']
        fill_option_combo(self.format_name, opt, 'default')
        QtCore.QObject.connect(self.format_other,
            QtCore.SIGNAL("toggled(bool)"), self._do_other_toggled)

        # Put the group box together
        group_layout = QtGui.QVBoxLayout()
        group_layout.addLayout(layout)
        group_layout.addWidget(self.format_name)
        group_box = QtGui.QGroupBox(gettext("Format"))
        group_box.setLayout(group_layout)
        return group_box

    def _do_other_toggled(self, value):
        self.format_name.setVisible(value)

    def _build_options_group_box(self):
        self.create_parent_dirs = QtGui.QCheckBox(
            gettext("Create parent directories of location, if required"))
        layout = QtGui.QVBoxLayout()
        layout.addWidget(self.create_parent_dirs)
        group_box = QtGui.QGroupBox(gettext("Options"))
        group_box.setLayout(layout)
        return group_box

    def get_model(self):
        if self.feature_branches.isChecked():
            return MODEL_FEATURE_BRANCHES
        elif self.shared_tree.isChecked():
            return MODEL_SHARED_TREE
        elif self.plain_branch.isChecked():
            return MODEL_PLAIN_BRANCH
        elif self.shared_repo.isChecked():
            return MODEL_SHARED_REPO
        elif self.shared_repo_no_trees.isChecked():
            return MODEL_SHARED_REPO_NO_TREES
        else:
            # XXX: raise an error here instead of assuming the default?
            return None

    def set_model(self, model):
        if model == MODEL_FEATURE_BRANCHES:
            self.feature_branches.setChecked(True)
        elif model == MODEL_SHARED_TREE:
            self.shared_tree.setChecked(True)
        elif model == MODEL_PLAIN_BRANCH:
            self.plain_branch.setChecked(True)
        elif model == MODEL_SHARED_REPO:
            self.shared_repo.setChecked(True)
        elif model == MODEL_SHARED_REPO_NO_TREES:
            self.shared_repo_no_trees.setChecked(True)
        else:
            raise UnknownWorkspaceModel(model)

    def get_location(self):
        # XXX: Should we insist on a location being set?
        return unicode(self.location.text()) or u'.'

    def get_format(self):
        if self.format_2.isChecked():
            return '2a'
        elif self.format_1.isChecked():
            return 'pack-0.92'
        else:
            return self.format_name.currentText()
        
    def do_start(self):
        args = []
        model = self.get_model()
        if model:
            args.append('--model')
            args.append(model)
        if self.create_parent_dirs.isChecked():
            args.append('--create-parent-dirs')
        format = self.get_format()
        if format:
            args.append('--format')
            args.append(format)
        args.append(self.get_location())
        self.process_widget.do_start(None, 'init-ala-explorer', *args)
